package com.example;

import akka.actor.typed.ActorRef;
import akka.actor.typed.Behavior;
import akka.actor.typed.javadsl.AbstractBehavior;
import akka.actor.typed.javadsl.ActorContext;
import akka.actor.typed.javadsl.Behaviors;
import akka.actor.typed.javadsl.Receive;

import java.util.ArrayList;

public class Queue extends AbstractBehavior<Queue.Message> {

    private final ArrayList<Integer> queue;

    public interface Message {};

    public record Get(ActorRef<Consumer.Message> replyTo) implements Message {  }
    public record Put(int i) implements Message {  }

    public static Behavior<Message> create() {
        return Behaviors.setup(Queue::new);
    }

    private Queue(ActorContext<Message> context) {
        super(context);
        this.queue = new ArrayList<Integer>();
    }

    @Override
    public Receive<Message> createReceive() {
        return newReceiveBuilder()
                .onMessage(Get.class, this::onGet)
                .onMessage(Put.class, this::onPut)
                .build();
    }

    private Behavior<Message> onPut(Put msg) {
        this.queue.add(msg.i);
        return this;
    }
    private Behavior<Message> onGet(Get msg) {
        if (this.queue.isEmpty()) {
            msg.replyTo.tell(Consumer.Error.EMPTY_QUEUE);
        } else {
            int i = this.queue.remove(0);
            msg.replyTo.tell(new Consumer.Reply(i));
        }
        return this;
    }
}
